package com.jeeplus.sys.controller;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.aliyun.oss.ServiceException;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.aop.demo.annotation.DemoMode;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.common.beanvalidator.BeanValidators;
import com.jeeplus.core.excel.utils.EasyPoiUtil;
import com.jeeplus.core.query.QueryWrapperGenerator;
import com.jeeplus.security.util.SecurityUtils;
import com.jeeplus.sys.constant.enums.ProjectStatusEnum;
import com.jeeplus.sys.constant.enums.ProjectTypeEnum;
import com.jeeplus.sys.constant.enums.YesOrNoEnum;
import com.jeeplus.sys.domain.User;
import com.jeeplus.sys.domain.UserProject;
import com.jeeplus.sys.service.UserProjectService;
import com.jeeplus.sys.service.UserService;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.service.dto.UserProjectDTO;
import com.jeeplus.sys.service.mapstruct.UserProjectWrapper;
import com.jeeplus.sys.service.mapstruct.UserWrapper;
import com.jeeplus.sys.utils.UserUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.time.DateUtils;
import org.dozer.inject.DozerBeanContainer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import javax.validation.Valid;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;


/**
 * 项目Controller
 *
 * @author lizhenhao
 * @version 2022-08-02
 */
@Api("项目管理")
@RestController
@RequestMapping(value = "/sys/project")
public class UserProjectController {

    @Autowired
    private UserProjectService userProjectService;

    @Autowired
    private UserService userService;

    private int successNum;

    @ApiLog("查询项目列表")
    @PreAuthorize("hasAuthority('sys:project:list')")
    @GetMapping("list")
    public ResponseEntity<IPage<UserProject>> data(UserProject userProject, Page<UserProject> page) throws Exception {
        QueryWrapper<UserProject> queryWrapper = QueryWrapperGenerator.buildQueryCondition (userProject, UserProject.class);
        queryWrapper.lambda().eq(StrUtil.isNotEmpty(userProject.getItemType()),UserProject::getItemType,userProject.getItemType());
        IPage<UserProject> result = userProjectService.selectPage (userProject);

        return ResponseEntity.ok (result);
    }

    /**
     * 查询项目数据
     * @param id
     * @return
     */
    @ApiLog("查询项目详情")
    @PreAuthorize ("hasAnyAuthority('sys:project:view','sys:project:add','sys:project:edit')")
    @GetMapping("queryById")
    public ResponseEntity queryById(@RequestParam("id") String id) {
        UserProject userProject = userProjectService.getById ( id );
        return ResponseEntity.ok (userProject);
    }

    /**
     * 保存项目信息
     * @param projectDTO
     * @return
     */
    @ApiLog("保存项目")
    @PreAuthorize ("hasAnyAuthority('sys:post:add','sys:post:edit')")
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody UserProjectDTO projectDTO){
        //新增或编辑表单保存
        UserProject project = UserProjectWrapper.INSTANCE.toEntity (projectDTO);
        UserProject pro = userProjectService.projectDtoToDate(project,projectDTO);
        pro.setItemType(projectDTO.getItemType());

        if(StrUtil.isNotEmpty(projectDTO.getProjectHead())){
            String userId = userService.getUserIdByLogin(projectDTO.getProjectHead());
            if(StrUtil.isNotEmpty(userId)){
                pro.setProjectHead(userId);
            }else{
                pro.setProjectHead("");
            }

        }

        if(StrUtil.isNotEmpty(projectDTO.getEvaluationPersonOne())){
            String userId = userService.getUserIdByLogin(projectDTO.getEvaluationPersonOne());
            if(StrUtil.isNotEmpty(userId)){
                pro.setEvaluationPersonOne(userId);
            }else{
                pro.setEvaluationPersonOne("");
            }
        }

        if(StrUtil.isNotEmpty(projectDTO.getEvaluationPersonTwo())){
            String userId = userService.getUserIdByLogin(projectDTO.getEvaluationPersonTwo());
            if(StrUtil.isNotEmpty(userId)){
                pro.setEvaluationPersonTwo(userId);
            }else{
                pro.setEvaluationPersonTwo("");
            }
        }

        if(StrUtil.isEmpty(projectDTO.getId())){

            //判断一下文号是否存在
            if(StrUtil.isNotEmpty(projectDTO.getDocumentNum())){
                if(ObjectUtil.isNotEmpty(userProjectService.selectByDocumentNum(projectDTO.getDocumentNum()))){
                    return ResponseEntity.badRequest().body("不可使用已存在的文号");
                }
            }

            userProjectService.save(pro);//新增
        }else{

            //判断一下文号是否存在
            if(StrUtil.isNotEmpty(projectDTO.getDocumentNum())){
                UserProject userProjects = userProjectService.selectByDocumentNum(projectDTO.getDocumentNum());
                if(ObjectUtil.isNotEmpty(userProjects)){
                    if(!projectDTO.getId().equals(userProjects.getId())){
                        return ResponseEntity.badRequest().body("不可使用已存在的文号");
                    }
                }
            }

            LambdaUpdateWrapper<UserProject> wrapper = userProjectService.getLambdaUpdateWrapper(projectDTO);
            userProjectService.update (pro,wrapper);//修改
        }

        return ResponseEntity.ok ("保存项目成功");
    }

    /**
     * 删除项目
     * @param ids
     * @return
     */
    @ApiLog("删除项目")
    @PreAuthorize ("hasAuthority('sys:project:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        String idArray[] =ids.split(",");
        userProjectService.removeByIds (Lists.newArrayList (idArray));
        return ResponseEntity.ok ("删除项目成功");
    }

    /**
     * 导入项目数据
     *
     * @return
     */
    @DemoMode
    @PreAuthorize("hasAnyAuthority('sys:project:import')")
    @PostMapping("import")
    @ApiOperation(value = "导入项目数据excel")
    public ResponseEntity importFile(MultipartFile file) {

        this.successNum = 0;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        ArrayList<UserProject> arrayList = new ArrayList<>();
        HashMap<String,String> hashMap = new HashMap<>();

        //评估项目sheet1
        List<UserProjectDTO> listA = EasyPoiUtil.importSheetExcel(file, 1, 1, 0,1, UserProjectDTO.class);

        String resultA = importDecide(listA, sdf, arrayList, hashMap, successNum, ProjectTypeEnum.ASSESS.getValue());
        if(StrUtil.isNotEmpty(resultA)){
            return ResponseEntity.badRequest().body  (resultA);
        }

        //咨询项目sheet2
        List<UserProjectDTO> listB = EasyPoiUtil.importSheetExcel(file, 1, 1, 1,1, UserProjectDTO.class);

        String resultB = importDecide(listB, sdf, arrayList, hashMap, successNum, ProjectTypeEnum.CONSULTATION.getValue());
        if(StrUtil.isNotEmpty(resultB)){
            return ResponseEntity.badRequest().body  (resultB);
        }

        if(hashMap.size() != listA.size() + listB.size()){
            return ResponseEntity.badRequest().body  ("文件中存在重复的文号或者多个空文号");
        }

        userProjectService.saveBatch(arrayList);

        return ResponseEntity.ok("已成功导入 " + successNum + " 条数据");
    }
    public String importDecide(List<UserProjectDTO> list,SimpleDateFormat sdf,ArrayList<UserProject> arrayList,HashMap<String,String> hashMap,int successNum,String itemType){

        for (UserProjectDTO project : list) {

            try {
                if(StrUtil.isNotEmpty(project.getEvaluationReportDateUi())){
                    String format = sdf.format(DateUtils.parseDate(project.getEvaluationReportDateUi(), new String[]{"yyyy/MM/dd"}));
                    project.setEvaluationReportDate(sdf.parse(format));
                    project.setStatus(ProjectStatusEnum.FINISH.getValue());
                }
            }catch (Exception e){
                if(StrUtil.isNotEmpty(project.getEvaluationReportDateUi())){
                    if(project.getEvaluationReportDateUi().contains("报告已作废")){
                        project.setStatus(ProjectStatusEnum.INVALID.getValue());
                        project.setAssessReportMessage(project.getEvaluationReportDateUi());
                    }else{
                        project.setStatus(ProjectStatusEnum.STAGING.getValue());
                        project.setAssessReportMessage(project.getEvaluationReportDateUi());
                    }
                }else{
                    project.setStatus(ProjectStatusEnum.STAGING.getValue());
                }
            }

            try {
                if(StrUtil.isNotEmpty(project.getEvaluationBaseDateUi())){
                    String format = sdf.format(DateUtils.parseDate(project.getEvaluationBaseDateUi(), new String[]{"yyyy/MM/dd"}));
                    project.setEvaluationBaseDate(sdf.parse(format));
                }
            }catch (Exception e){
                if(StrUtil.isNotEmpty(project.getEvaluationBaseDateUi()))
                    project.setAssessBaseMessage(project.getEvaluationBaseDateUi());
            }

            try {
                if(StrUtil.isNotEmpty(project.getInvoiceDateUi())){
                    String format = sdf.format(DateUtils.parseDate(project.getInvoiceDateUi(), new String[]{"yyyy/MM/dd"}));
                    project.setInvoiceDate(sdf.parse(format));
                }
            }catch (Exception e){
                if(StrUtil.isNotEmpty(project.getInvoiceDateUi()))
                    project.setInvoiceMessage(project.getInvoiceDateUi());
            }

            try {
                if(StrUtil.isNotEmpty(project.getReimbursementDateUi())){
                    String format = sdf.format(DateUtils.parseDate(project.getReimbursementDateUi(), new String[]{"yyyy/MM/dd"}));
                    project.setReimbursementDate(sdf.parse(format));
                }
            }catch (Exception e){
                e.printStackTrace();
            }

            //项目负责人、签字评估师1、签字评估师2 关联user表的id
            if(StrUtil.isNotEmpty(project.getProjectHead())){
                String id = userService.getUserIdByLogin(project.getProjectHead());
                if(StrUtil.isNotEmpty(id)){
                    project.setProjectHead(id);
                }else{
                    project.setProjectHead("");
                }
            }

            if(StrUtil.isNotEmpty(project.getEvaluationPersonOne())){
                String id = userService.getUserIdByLogin(project.getEvaluationPersonOne());
                if(ObjectUtil.isNotEmpty(id)){
                    project.setEvaluationPersonOne(id);
                }else{
                    project.setEvaluationPersonOne("");
                }
            }

            if(StrUtil.isNotEmpty(project.getEvaluationPersonTwo())){
                String id = userService.getUserIdByLogin(project.getEvaluationPersonTwo());
                if(ObjectUtil.isNotEmpty(id)){
                    project.setEvaluationPersonTwo(id);
                }else{
                    project.setEvaluationPersonTwo("");
                }
            }

            //文号的问题
            if(StrUtil.isNotEmpty(project.getDocumentNum())) {
                UserProject userProject = userProjectService.selectByDocumentNum(project.getDocumentNum());
                if(ObjectUtil.isNotEmpty(userProject)){
                    return "文件中存在已在档的文号";
                }
            }

            //是否开票
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsInvoice())){
                project.setIsInvoice(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsInvoice())){
                project.setIsInvoice(YesOrNoEnum.NO.getValue());
            }

            //合同是否存档
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsContractArchive())){
                project.setIsContractArchive(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsContractArchive())){
                project.setIsContractArchive(YesOrNoEnum.NO.getValue());
            }

            //底稿是否完好
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsPapersIntact())){
                project.setIsPapersIntact(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsPapersIntact())){
                project.setIsPapersIntact(YesOrNoEnum.NO.getValue());
            }

            //底稿是否归档
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsPapersArchive())){
                project.setIsPapersArchive(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsPapersArchive())){
                project.setIsPapersArchive(YesOrNoEnum.NO.getValue());
            }

            //外勤是否已经报销
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsOpsReimbursement())){
                project.setIsOpsReimbursement(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsOpsReimbursement())){
                project.setIsOpsReimbursement(YesOrNoEnum.NO.getValue());
            }

            //是否已经报销提成
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsCommissionReimbursement())){
                project.setIsCommissionReimbursement(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsCommissionReimbursement())){
                project.setIsCommissionReimbursement(YesOrNoEnum.NO.getValue());
            }

            hashMap.put(project.getDocumentNum(),null);

            UserProject userProject = UserProjectWrapper.INSTANCE.toEntity(project);
            //项目类型 1.评估项目  2.咨询项目
            userProject.setItemType(itemType);
            arrayList.add(userProject);
            this.successNum++;
        }

        return null;
    }


    /**
     * 下载导入项目数据模板
     *
     * @param response
     * @return
     */
    @PreAuthorize ("hasAnyAuthority('sys:project:import')")
    @GetMapping("import/template/assess")
    @ApiOperation(value = "下载模板")
    public void importFileTemplateAssess(HttpServletResponse response) {
        String fileName = "项目数据导入模板.xlsx";
        List<UserProjectDTO> list = Lists.newArrayList();
        EasyPoiUtil.exportExcel ( list, "2021年资产评估项目备查登记表",  "项目明细表", UserProjectDTO.class, fileName, response );
    }


    /**
     * 下载导入项目数据模板
     *
     * @param response
     * @return
     */
    @PreAuthorize ("hasAnyAuthority('sys:project:import')")
    @GetMapping("import/template/consultation")
    @ApiOperation(value = "下载模板")
    public void importFileTemplateConsultation(HttpServletResponse response) {
        String fileName = "项目数据导入模板.xlsx";
        List<UserProjectDTO> list = Lists.newArrayList();
        EasyPoiUtil.exportExcel ( list, "2021年资产评估项目备查登记表",  "咨询报告", UserProjectDTO.class, fileName, response );
    }

}
